==> monitor-0.1.tar.gz: monitor/libraries/user.php <==
<?php
	/* libraries/user.php
	 *
	 * Copyright (C) by Hugo Leisink <hugo@leisink.net>
	 * This file is part of the Banshee PHP framework
	 * http://www.hiawatha-webserver.org/banshee
	 *
	 * Don't change this file, unless you know what you are doing.
	 */

	class user {
		private $db = null;
		private $id = null;
		private $client_ip = null;
		private $logged_in = false;
		private $record = array();
		private $is_admin = false;

		/* Constructor
		 *
		 * INPUT:  object database
		 * OUTPUT: -
		 * ERROR:  -
		 */
		public function __construct($db) {
			$this->db = $db;

			$this->client_ip = $_SERVER["REMOTE_ADDR"];
			if (isset($_SERVER["HTTP_X_FORWARDED_FOR"])) {
				$this->client_ip .= "/".$_SERVER["HTTP_X_FORWARDED_FOR"];
			}

			if (isset($_SESSION["user_id"])) {
				if (time() - $_SESSION["last_private_visit"] >= SESSION_TIMEOUT) {
					unset($_SESSION["user_id"]);
				} else if (($_SESSION["binded_ip"] === NO) || ($_SESSION["binded_ip"] === $this->client_ip)) {
					$this->load_user_record($_SESSION["user_id"]);
				}
			}
		}

		/* Magic method get
		 *
		 * INPUT:  string key
		 * OUTPUT: mixed value
		 * ERROR:  null
		 */
		public function __get($var) {
			if ($var == "logged_in") {
				return $this->logged_in;
			} else if ($var == "id") {
				return $this->id;
			} else if ($var == "is_admin") {
				return $this->is_admin;
			} else if ($var == "client_ip") {
				return $this->client_ip;
			} else if (isset($this->record[$var])) {
				return $this->record[$var];
			}

			return null;
		}

		/* Store user information from database in $this->record
		 *
		 * INPUT:  int user identifier
		 * OUTPUT: -
		 * ERROR:  -
		 */
		private function load_user_record($user_id) {
			if (($this->record = $this->db->entry("users", $user_id)) == false) {
				$this->logout();
			} else if ($this->record["status"] == USER_STATUS_DISABLED) {
				$this->logout();
			} else {
				$this->id = $user_id;
				$this->logged_in = true;

				$this->record["role_ids"] = array();
				$query = "select role_id from user_role where user_id=%d";
				if (($roles = $this->db->execute($query, $this->record["id"])) != false) {
					foreach ($roles as $role) {
						array_push($this->record["role_ids"], $role["role_id"]);
						if ((int)$role["role_id"] === (int)ADMIN_ROLE_ID) {
							$this->is_admin = true;
						}
					}
				}
			}
		}

		/* Verify user credentials
		 *
		 * INPUT:  string username, string password, boolean challenge-response method used
		 * OUTPUT: boolean login correct
		 * ERROR:  -
		 */
		public function login_password($username, $password, $use_challenge_response_method) {
			$query = "select * from users where username=%s and status!=%d limit 1";
			if (($data = $this->db->execute($query, $username, USER_STATUS_DISABLED)) != false) {
				$user = $data[0];

				if ($use_challenge_response_method) {
					if (md5($_SESSION["challenge"].$user["password"]) === $password) {
						$this->logged_in = true;
					}
				} else {
					if ($user["password"] === md5($password)) {
						$this->logged_in = true;
					}
				}

				if ($this->logged_in) {
					$this->load_user_record((int)$user["id"]);
					$this->log_action("user logged-in");

					$_SESSION["user_id"] = $this->id;
					$_SESSION["binded_ip"] = NO;
					$_SESSION["last_private_visit"] = time();
				}
			}

			return $this->logged_in;
		}

		/* Verify one time key
		 *
		 * INPUT:  string one time key
		 * OUTPUT: boolean key valid
		 * ERROR:  -
		 */
		public function login_one_time_key($key) {
			if ($key == "") {
				return false;
			}

			if (($user = $this->db->entry("users", $key, "one_time_key")) == false) {
				return false;
			}

			$query = "update users set one_time_key=null where id=%d";
			$this->db->query($query, $user["id"]);

			$this->logged_in = true;
			$this->load_user_record($user["id"]);
			$this->log_action("user logged-in");

			$_SESSION["user_id"] = $user["id"];
			$_SESSION["binded_ip"] = NO;
			$_SESSION["last_private_visit"] = time();

			return true;
		}

		/* Logout current user
		 *
		 * INPUT:  -
		 * OUTPUT: -
		 * ERROR:  -
		 */
		public function logout() {
			$this->log_action("user logged-out");

			session_destroy();
			unset($_SESSION);
			session_name(SESSION_NAME);
			session_start();

			$this->id = null;
			$this->logged_in = false;
			$this->record = array();
			$this->is_admin = false;
		}

		/* Checks if user has access to page
		 *
		 * INPUT:  string page identifier
		 * OUTPUT: boolean user has access to page
		 * ERROR:  -
		 */
		public function access_allowed($page) {
			/* Always access
			 */
			$allowed = array("logout");
			if ($this->is_admin || in_array($page, $allowed)) {
				return true;
			}

			/* Public page
			 */
			if (in_array($page, config_file("public_pages"))) {
				return true;
			}

			/* No roles, no access
			 */
			if (count($this->role_ids) == 0) {
				return false;
			}

			/* Check access
			 */
			if (in_array($page, config_file("private_pages"))) {
				/* Pages on disk (modules)
				 */
				$conditions = $rids = array();
				foreach ($this->role_ids as $rid) {
					array_push($conditions, "id=%d");
					array_push($rids, $rid);
				}

				$query = "select %S from roles where ".implode(" or ", $conditions);
				if (($access = $this->db->execute($query, $page, $rids)) == false) {
					return false;
				}
			} else {
				/* Pages in database
				 */
				$conditions = $rids = array();
				foreach ($this->role_ids as $rid) {
					array_push($conditions, "a.role_id=%d");
					array_push($rids, $rid);
				}

				$query = "select a.level from page_access a, pages p ".
				         "where a.page_id=p.id and p.url=%s and a.level>0 ".
				         "and (".implode(" or ", $conditions).")";
				if (($access = $this->db->execute($query, "/".$page, $rids)) == false) {
					return false;
				}
			}

			return max(array_flatten($access)) > 0;
		}

		/* Bind current session to IP address
		 *
		 * INPUT:  -
		 * OUTPUT: -
		 * ERROR:  -
		 */
		public function bind_to_ip() {
			$_SESSION["binded_ip"] = $this->client_ip;
		}

		/* Verify if user has a certain role
		 *
		 * INPUT:  int role identifier / string role name
		 * OUTPUT: boolean user has role
		 * ERROR:  -
		 */
		public function has_role($role) {
			if (is_int($role)) {
				return in_array($role, $this->role_ids);
			} else if (is_string($role)) {
				if (($entry = $this->db->entry("roles", $role, "name")) != false) {
					return $this->has_role((int)$entry["id"]);
				}
			} else if (is_array($role)) {
				foreach ($role as $item) {
					if ($this->has_role($item)) {	
						return true;
					}
				}
			}

			return false;
		}

		/* Log user action
		 *
		 * INPUT:  string action
		 * OUTPUT: -
		 * ERROR:  -
		 */
		public function log_action($action) {
			$mesg = $_SERVER["REMOTE_ADDR"]."|".date("D d M Y H:i:s")."|";
			if ($this->logged_in == false) {
				$mesg .= "-";
			} else if (isset($_SESSION["user_switch"]) == false) {
				$mesg .= $this->id;
			} else {
				$mesg .= $_SESSION["user_switch"].":".$this->id;
			}
			$mesg .= "|".$action."\n";

			if (($fp = fopen("../logfiles/actions.log", "a")) != false) {
				fputs($fp, $mesg);
				fclose($fp);
			}
		}
	}
?>

==> monitor-0.2.tar.gz: monitor/libraries/user.php <==
<?php
	/* libraries/user.php
	 *
	 * Copyright (C) by Hugo Leisink <hugo@leisink.net>
	 * This file is part of the Banshee PHP framework
	 * http://www.hiawatha-webserver.org/banshee
	 *
	 * Don't change this file, unless you know what you are doing.
	 */

	class user {
		private $db = null;
		private $id = null;
		private $client_ip = null;
		private $logged_in = false;
		private $record = array();
		private $is_admin = false;

		/* Constructor
		 *
		 * INPUT:  object database
		 * OUTPUT: -
		 * ERROR:  -
		 */
		public function __construct($db) {
			$this->db = $db;

			$this->client_ip = $_SERVER["REMOTE_ADDR"];
			if (isset($_SERVER["HTTP_X_FORWARDED_FOR"])) {
				$this->client_ip .= "/".$_SERVER["HTTP_X_FORWARDED_FOR"];
			}

			if (isset($_SESSION["user_id"])) {
				if (time() - $_SESSION["last_private_visit"] >= SESSION_TIMEOUT) {
					unset($_SESSION["user_id"]);
				} else if (($_SESSION["binded_ip"] === NO) || ($_SESSION["binded_ip"] === $this->client_ip)) {
					$this->load_user_record($_SESSION["user_id"]);
				}
			}
		}

		/* Magic method get
		 *
		 * INPUT:  string key
		 * OUTPUT: mixed value
		 * ERROR:  null
		 */
		public function __get($var) {
			if ($var == "logged_in") {
				return $this->logged_in;
			} else if ($var == "id") {
				return $this->id;
			} else if ($var == "is_admin") {
				return $this->is_admin;
			} else if ($var == "client_ip") {
				return $this->client_ip;
			} else if (isset($this->record[$var])) {
				return $this->record[$var];
			}

			return null;
		}

		/* Store user information from database in $this->record
		 *
		 * INPUT:  int user identifier
		 * OUTPUT: -
		 * ERROR:  -
		 */
		private function load_user_record($user_id) {
			if (($this->record = $this->db->entry("users", $user_id)) == false) {
				$this->logout();
			} else if ($this->record["status"] == USER_STATUS_DISABLED) {
				$this->logout();
			} else {
				$this->id = $user_id;
				$this->logged_in = true;

				$this->record["role_ids"] = array();
				$query = "select role_id from user_role where user_id=%d";
				if (($roles = $this->db->execute($query, $this->record["id"])) != false) {
					foreach ($roles as $role) {
						array_push($this->record["role_ids"], $role["role_id"]);
						if ((int)$role["role_id"] === (int)ADMIN_ROLE_ID) {
							$this->is_admin = true;
						}
					}
				}
			}
		}

		/* Verify user credentials
		 *
		 * INPUT:  string username, string password, boolean challenge-response method used
		 * OUTPUT: boolean login correct
		 * ERROR:  -
		 */
		public function login_password($username, $password, $use_challenge_response_method) {
			$query = "select * from users where username=%s and status!=%d limit 1";
			if (($data = $this->db->execute($query, $username, USER_STATUS_DISABLED)) != false) {
				$user = $data[0];

				if ($use_challenge_response_method) {
					if (md5($_SESSION["challenge"].$user["password"]) === $password) {
						$this->logged_in = true;
					}
				} else {
					if ($user["password"] === md5($password)) {
						$this->logged_in = true;
					}
				}

				if ($this->logged_in) {
					$this->load_user_record((int)$user["id"]);
					$this->log_action("user logged-in");

					$_SESSION["user_id"] = $this->id;
					$_SESSION["binded_ip"] = NO;
					$_SESSION["last_private_visit"] = time();
				}
			}

			return $this->logged_in;
		}

		/* Verify one time key
		 *
		 * INPUT:  string one time key
		 * OUTPUT: boolean key valid
		 * ERROR:  -
		 */
		public function login_one_time_key($key) {
			if ($key == "") {
				return false;
			}

			if (($user = $this->db->entry("users", $key, "one_time_key")) == false) {
				return false;
			}

			$query = "update users set one_time_key=null where id=%d";
			$this->db->query($query, $user["id"]);

			$this->logged_in = true;
			$this->load_user_record($user["id"]);
			$this->log_action("user logged-in");

			$_SESSION["user_id"] = $user["id"];
			$_SESSION["binded_ip"] = NO;
			$_SESSION["last_private_visit"] = time();

			return true;
		}

		/* Logout current user
		 *
		 * INPUT:  -
		 * OUTPUT: -
		 * ERROR:  -
		 */
		public function logout() {
			$this->log_action("user logged-out");

			session_destroy();
			unset($_SESSION);
			session_name(SESSION_NAME);
			session_start();

			$this->id = null;
			$this->logged_in = false;
			$this->record = array();
			$this->is_admin = false;
		}

		/* Checks if user has access to page
		 *
		 * INPUT:  string page identifier
		 * OUTPUT: boolean user has access to page
		 * ERROR:  -
		 */
		public function access_allowed($page) {
			/* Always access
			 */
			$allowed = array("logout");
			if ($this->is_admin || in_array($page, $allowed)) {
				return true;
			}

			/* Public page
			 */
			if (in_array($page, page_to_module(config_file("public_pages")))) {
				return true;
			}

			/* No roles, no access
			 */
			if (count($this->role_ids) == 0) {
				return false;
			}

			/* Check access
			 */
			if (in_array($page, page_to_module(config_file("private_pages")))) {
				/* Pages on disk (modules)
				 */
				$conditions = $rids = array();
				foreach ($this->role_ids as $rid) {
					array_push($conditions, "id=%d");
					array_push($rids, $rid);
				}

				$query = "select %S from roles where ".implode(" or ", $conditions);
				if (($access = $this->db->execute($query, $page, $rids)) == false) {
					return false;
				}
			} else {
				/* Pages in database
				 */
				$conditions = $rids = array();
				foreach ($this->role_ids as $rid) {
					array_push($conditions, "a.role_id=%d");
					array_push($rids, $rid);
				}

				$query = "select a.level from page_access a, pages p ".
				         "where a.page_id=p.id and p.url=%s and a.level>0 ".
				         "and (".implode(" or ", $conditions).")";
				if (($access = $this->db->execute($query, "/".$page, $rids)) == false) {
					return false;
				}
			}

			return max(array_flatten($access)) > 0;
		}

		/* Bind current session to IP address
		 *
		 * INPUT:  -
		 * OUTPUT: -
		 * ERROR:  -
		 */
		public function bind_to_ip() {
			$_SESSION["binded_ip"] = $this->client_ip;
		}

		/* Verify if user has a certain role
		 *
		 * INPUT:  int role identifier / string role name
		 * OUTPUT: boolean user has role
		 * ERROR:  -
		 */
		public function has_role($role) {
			if (is_int($role)) {
				return in_array($role, $this->role_ids);
			} else if (is_string($role)) {
				if (($entry = $this->db->entry("roles", $role, "name")) != false) {
					return $this->has_role((int)$entry["id"]);
				}
			} else if (is_array($role)) {
				foreach ($role as $item) {
					if ($this->has_role($item)) {	
						return true;
					}
				}
			}

			return false;
		}

		/* Log user action
		 *
		 * INPUT:  string action
		 * OUTPUT: -
		 * ERROR:  -
		 */
		public function log_action($action) {
			$mesg = $_SERVER["REMOTE_ADDR"]."|".date("D d M Y H:i:s")."|";
			if ($this->logged_in == false) {
				$mesg .= "-";
			} else if (isset($_SESSION["user_switch"]) == false) {
				$mesg .= $this->id;
			} else {
				$mesg .= $_SESSION["user_switch"].":".$this->id;
			}
			$mesg .= "|".$action."\n";

			if (($fp = fopen("../logfiles/actions.log", "a")) != false) {
				fputs($fp, $mesg);
				fclose($fp);
			}
		}
	}
?>
